package fr.username404.snowygui.gui.elements

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.FontUtil
import fr.username404.snowygui.gui.Renderable.Rendering.defaultRectFunc
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import net.minecraft.client.gui.components.events.GuiEventListener

class ClickButton(
    x: Double = 0.0, y: Double = 0.0,
    width: Int = 73, height: Int = 8,
    color: Int, private val kind: Type = Type.TOGGLE,
    private val Title: String? = null,
    private val action: (() -> Unit)? = null,
): ColoredElement(x, y, width, height, color, 0.60F), GuiEventListener {
    companion object {
        enum class Type {
            TOGGLE,
            CLICK
        }
        internal var lightningFactor: Float = 0.33F
    }
    fun execAction() {
        runBlocking {
            action?.let {
                launch {
                    it.invoke()
                }
            }
        }
    }
    private var wasWithinBounds: Boolean = false
    private var toggled: Boolean = false
    private fun lightUp() { opacity += lightningFactor }
    private fun lightDown() { opacity -= lightningFactor }
    override fun mouseClicked(d: Double, e: Double, i: Int): Boolean {
        wasWithinBounds = withinBounds(d, e).also {
            if (it) {
                if (!toggled) lightUp() else lightDown()
                if ((kind == Type.TOGGLE)) {
                    toggled = !toggled
                    execAction()
                }
            }
        }
        return false
    }

    override fun mouseReleased(d: Double, e: Double, i: Int): Boolean {
        if (wasWithinBounds && (kind != Type.TOGGLE)) {
            lightDown()
            execAction()
        }; return false
    }
    override fun render(poseStack: PoseStack?) {
        RenderSystem.disableTexture()
        RenderSystem.enableBlend()
        defaultRectFunc()
        RenderSystem.enableTexture()
        RenderSystem.disableBlend()
        if (Title != null && poseStack != null) {
            FontUtil.drawScaled(poseStack, Title, x + 1, y + 1, 0.75F)
        }
    }
}